/**
 * @brief Test for NOBLOCK operations
 *
 * @file atmiclt18.c
 */
/* -----------------------------------------------------------------------------
 * Enduro/X Middleware Platform for Distributed Transaction Processing
 * Copyright (C) 2009-2016, ATR Baltic, Ltd. All Rights Reserved.
 * Copyright (C) 2017-2019, Mavimax, Ltd. All Rights Reserved.
 * This software is released under one of the following licenses:
 * AGPL (with Java and Go exceptions) or Mavimax's license for commercial use.
 * See LICENSE file for full text.
 * -----------------------------------------------------------------------------
 * AGPL license:
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Affero General Public License, version 3 as published
 * by the Free Software Foundation;
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
 * PARTICULAR PURPOSE. See the GNU Affero General Public License, version 3
 * for more details.
 *
 * You should have received a copy of the GNU Affero General Public License along 
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 * -----------------------------------------------------------------------------
 * A commercial use license is available from Mavimax, Ltd
 * contact@mavimax.com
 * -----------------------------------------------------------------------------
 */
#include <string.h>
#include <stdio.h>
#include <stdlib.h>
#include <memory.h>
#include <math.h>
#include <unistd.h>

#include <atmi.h>
#include <ubf.h>
#include <ndebug.h>
#include <test.fd.h>
#include <ndrstandard.h>
#include <nstopwatch.h>
/*---------------------------Externs------------------------------------*/
/*---------------------------Macros-------------------------------------*/
/*---------------------------Enums--------------------------------------*/
/*---------------------------Typedefs-----------------------------------*/
/*---------------------------Globals------------------------------------*/
/*---------------------------Statics------------------------------------*/
/*---------------------------Prototypes---------------------------------*/

/*
 * Do the test call to the server
 */
int main(int argc, char** argv) {

    UBFH *p_ub = (UBFH *)tpalloc("UBF", NULL, 9218);
    long rsplen;
    int i;
    int ret=EXSUCCEED;
    int cd_got;
    int cd[3];
    int got_send_block;
    char bigmsg[8000];
    
    Badd(p_ub, T_STRING_FLD, "THIS IS TEST FIELD 1", 0);
    Badd(p_ub, T_STRING_FLD, "THIS IS TEST FIELD 2", 0);
    Badd(p_ub, T_STRING_FLD, "THIS IS TEST FIELD 3", 0);
    
    cd[0] = tpacall("TESTSV", (char *)p_ub, 0L, 0L);
    if (cd[0] <= 0)
    {
        NDRX_LOG(log_error, "TESTERROR first tpacall got %d: %s",
            cd[0], tpstrerror(tperrno));
        EXFAIL_OUT(ret);
    }
    
    cd[1] = tpacall("TESTSV", (char *)p_ub, 0L, 0L);
    if (cd[1] <= 0)
    {
        NDRX_LOG(log_error, "TESTERROR second tpacall got %d: %s",
            cd[1], tpstrerror(tperrno));
        EXFAIL_OUT(ret);
    }
    
    cd[2] = tpacall("TESTSV", (char *)p_ub, 0L, 0L);
    if (cd[2] <= 0)
    {
        NDRX_LOG(log_error, "TESTERROR third tpacall got %d: %s",
            cd[2], tpstrerror(tperrno));
        EXFAIL_OUT(ret);
    }
    
    /* wait for tout */
    sleep(10);
    
    for (i = 0; i<3; i++)
    {
        cd_got = cd[i];
        if (EXSUCCEED==(ret = tpgetrply(&cd_got, (char **)&p_ub, &rsplen, TPNOBLOCK)))
        {
            NDRX_LOG(log_error, "TESTERROR The call was ok to server "
                    "- must be tout!");
            ret=EXFAIL;
            goto out;
        }

        ret = EXSUCCEED;
        if (cd_got != cd[i])
        {
            NDRX_LOG(log_error, "TESTERROR cd %d <> go_cd %d", cd, cd_got);
            ret=EXFAIL;
            goto out;
        }

        if (tperrno!=TPETIME)
        {
            NDRX_LOG(log_error, "Te error should be time-out!");
            ret=EXFAIL;
            goto out;
        }
    }
    
    /* the the case when we get reply back, this should not be a timeout. */
    cd[0] = tpacall("ECHO", (char *)p_ub, 0L, 0L);
    sleep(1);
    if (EXSUCCEED==(ret = tpgetrply(&cd_got, (char **)&p_ub, &rsplen, TPNOBLOCK | TPGETANY)))
    {
        NDRX_LOG(log_error, "TESTERROR Must be TPEBLOCK");
        ret=EXFAIL;
        goto out;
    }
    
    if (tperrno!=TPEBLOCK)
    {
        NDRX_LOG(log_error, "TESTERROR! got tperror=%d %s expected TPENOBLOCK %d", 
                tperrno, tpstrerror(tperrno), TPEBLOCK);
        ret=EXFAIL;
        goto out;
    }
    
    sleep(10);
    if (EXSUCCEED!=(ret = tpgetrply(&cd_got, (char **)&p_ub, &rsplen, TPNOBLOCK | TPGETANY)))
    {
        NDRX_LOG(log_error, "TESTERROR Normal reply fails!!!");
        ret=EXFAIL;
        goto out;
    }
    
    if (cd_got!=cd[0])
    {
        NDRX_LOG(log_error, "TESTERROR Got invalid response! "
                "cd[0] = %d got_cd = %d", cd[0], cd_got);
        ret=EXFAIL;
        goto out;
    }
    
    /* test for noblock op -> queue empty... */    
    if (EXSUCCEED==(ret = tpgetrply(&cd_got, (char **)&p_ub, &rsplen, TPNOBLOCK)))
    {
        NDRX_LOG(log_error, "TESTERROR Queue must be empty!");
        ret=EXFAIL;
        goto out;
    }
    
    NDRX_LOG(log_debug, "got tperror=%d %s", tperrno, tpstrerror(tperrno));
    
    if (tperrno!=TPEBADDESC)
    {
        NDRX_LOG(log_error, "TESTERROR! got tperror=%d %s expected TPEBADDESC %d", 
                tperrno, tpstrerror(tperrno), TPEBADDESC);
        ret=EXFAIL;
        goto out;
    }
    ret=EXSUCCEED;
    
    /* Test for full service queue, we shall get TPEBLOCK back */
    
    if (EXSUCCEED!=Bchg(p_ub, T_CARRAY_FLD, 0, bigmsg, sizeof(bigmsg)))
    {
        NDRX_LOG(log_error, "TESTERROR! Failed to add bigmsg: %s", 
                Bstrerror(Berror));
        EXFAIL_OUT(ret);
    }
    
    got_send_block = EXFALSE;
    while (1)
    {
        if (EXFAIL==tpacall("BLOCKY", (char *)p_ub, 0L, TPNOBLOCK))
        {
            if (tperrno!=TPEBLOCK)
            {
                NDRX_LOG(log_error, "TESTERROR! got tperror=%d %s expected TPENOBLOCK %d", 
                        tperrno, tpstrerror(tperrno), TPEBLOCK);
                ret=EXFAIL;
                goto out;
            }
            else
            {
                got_send_block = EXTRUE;
                break;
            }
        }
    }
    
    if (!got_send_block)
    {
        NDRX_LOG(log_error, "TESTERROR: expected TPEBLOCK condition in previous run!");
        ret=EXFAIL;
        goto out;
    }
    
out:

    if (NULL!=p_ub)
    {
        tpfree((char *)p_ub);
    }

    tpterm();
    NDRX_LOG(log_error, "RESULT: %d", ret);
    return ret;
}

/* vim: set ts=4 sw=4 et smartindent: */
